"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EventBusValidator = exports.InvalidPublishTopic = void 0;
const tslib_1 = require("tslib");
// import { metrics } from "./eventbusMetrics";
const graphql_1 = require("graphql");
const validator_1 = require("./validator");
const eventbus_utils_1 = require("./eventbus-utils");
// interface Metrics {
//   consume: (topicName: string) => any;
//   consumeStart: (topicName: string) => () => any;
//   consumeError: (topicName: string) => any;
//   publish: (topicName: string) => any;
//   publishError: (topicName: string) => any;
// }
class InvalidPublishTopic extends Error {
    constructor(...params) {
        // Pass remaining arguments (including vendor specific ones) to parent constructor
        super(...params);
    }
}
exports.InvalidPublishTopic = InvalidPublishTopic;
/**
 *
 * Eventbus Class
 * @class EventBus
 *
 * @param {GraphQLSchema} publishSchema The GraphQL schema of message publisher to validated the published messaged agains
 */
class EventBusValidator {
    constructor(props) {
        this.getDocumentForTopic = (topic) => {
            return this.consumerTopics[topic];
        };
        this.validateConsumerQueries = (queries) => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                yield (0, eventbus_utils_1.validateQueries)(this.publisherSchema, [queries]);
                this.consumerTopics = (0, eventbus_utils_1.getTopicsFromDocument)(queries);
                return Object.keys(this.consumerTopics);
            }
            catch (e) {
                const graphqlSchemaObj = (0, graphql_1.printSchema)(this.publisherSchema);
                console.info(`Using schema`, graphqlSchemaObj);
                throw e;
            }
        });
        this.extractData = (args) => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            const queriedData = yield this.validator.extract(this.consumerTopics[args.topic], args.topic, args.data);
            if (queriedData.errors) {
                throw new Error(JSON.stringify(queriedData.errors, null, 2));
            }
            if (!queriedData.data) {
                throw new Error("Queried data is null");
            }
            return queriedData.data[args.topic];
        });
        this.publishValidate = (props) => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            if (!this.validator || !this.publishTopicNames) {
                throw new Error("You must specify publishSchema in the constructor");
            }
            if (this.publishTopicNames.indexOf(props.topic) === -1) {
                throw new InvalidPublishTopic(`Publish topic ${props.topic} has not been defined in the publishSchema ${this.publishTopicNames}`);
            }
            this.validator.validate(props.topic, props.payload);
        });
        this.publisherSchema = props.publisherSchema;
        this.validator = new validator_1.Validator(this.publisherSchema);
        this.publishTopicNames = (0, eventbus_utils_1.getRootQueryFields)(this.publisherSchema);
    }
}
exports.EventBusValidator = EventBusValidator;
