import { EventBusSubscriberCb } from "./EventBus";
import { DocumentNode, GraphQLSchema } from "graphql";
export declare type DataCb = (args: {
    baggage: Baggage;
    topic: string;
}) => Promise<unknown>;
interface SubscriptionConfig {
    queries: DocumentNode;
    schema: GraphQLSchema;
    cb: EventBusSubscriberCb;
    subscribe: (topics: string[], cb: DataCb) => Promise<unknown> | unknown | void;
}
export interface Baggage {
    payload: {};
    metadata: Metadata;
}
declare type OptionalPromise<T> = T | Promise<T> | undefined | null | void;
export declare type ConsumeEndHook = () => OptionalPromise<unknown>;
export declare type ConsumeSuccessHook = () => OptionalPromise<unknown>;
export declare type ConsumeErrorHook = (error: Error) => OptionalPromise<unknown>;
export declare type ConsumeStartHook = (args: {
    topic: string;
    fullData: {};
    documentNode: DocumentNode;
    metadata: Metadata;
}) => OptionalPromise<{
    consumeEndHook?: ConsumeEndHook;
    consumeErrorHook?: ConsumeErrorHook;
    consumeSuccessHook?: ConsumeSuccessHook;
}>;
export declare type PublishEndHook = () => OptionalPromise<unknown>;
export declare type PublishSuccessHook = () => OptionalPromise<unknown>;
export declare type PublishErrorHook = (error: Error) => OptionalPromise<unknown>;
export declare type PublishStartHook = (args: {
    topic: string;
    payload: {};
    metadata: Metadata;
}) => OptionalPromise<{
    publishEndHook?: PublishEndHook;
    publishErrorHook?: PublishErrorHook;
    publishSuccessHook?: PublishSuccessHook;
}>;
export interface EventBusPlugin {
    consumeStartHook?: ConsumeStartHook;
    publishStartHook?: PublishStartHook;
}
export interface Metadata {
    "x-request-id": string;
    publishTime: string;
    messageId: string;
    [key: string]: string;
}
export declare class VanillaEventBus {
    config: {
        publisher?: {
            schema: GraphQLSchema;
            publishInit?: (topics: string[]) => Promise<unknown>;
            publish: (args: {
                topic: string;
                baggage: Baggage;
            }) => Promise<unknown>;
            allowInvalidTopic?: boolean;
        };
        subscriber?: SubscriptionConfig;
        plugins?: EventBusPlugin[];
    };
    private consumeValidator;
    private publishValidator;
    constructor(config: {
        publisher?: {
            schema: GraphQLSchema;
            publishInit?: (topics: string[]) => Promise<unknown>;
            publish: (args: {
                topic: string;
                baggage: Baggage;
            }) => Promise<unknown>;
            allowInvalidTopic?: boolean;
        };
        subscriber?: SubscriptionConfig;
        plugins?: EventBusPlugin[];
    });
    init: () => Promise<void>;
    private consume;
    publish: (props: {
        topic: string;
        payload: {};
        metadata?: Partial<Metadata>;
    }) => Promise<void>;
}
export {};
