"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.VanillaEventBus = void 0;
const tslib_1 = require("tslib");
const EventBusValidator_1 = require("./EventBusValidator");
const eventbus_utils_1 = require("./eventbus-utils");
const _1 = require(".");
const uuid_1 = require("uuid");
const ts_is_present_1 = require("ts-is-present");
class VanillaEventBus {
    constructor(config) {
        this.config = config;
        this.init = () => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            if (this.config.publisher && this.config.publisher.publishInit) {
                const topicnames = (0, eventbus_utils_1.getRootQueryFields)(this.config.publisher.schema);
                yield this.config.publisher.publishInit(topicnames);
            }
            if (this.config.subscriber) {
                yield this.consume(this.config.subscriber);
            }
        });
        this.consume = (args) => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            var _a;
            if (!this.consumeValidator) {
                throw new Error("Subscriber config not added");
            }
            const consumeHooks = ((_a = this.config.plugins) === null || _a === void 0 ? void 0 : _a.map((a) => a.consumeStartHook).filter(ts_is_present_1.isPresent)) || [];
            yield this.consumeValidator
                .validateConsumerQueries(args.queries)
                .then((topics) => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
                yield args.subscribe(topics, ({ baggage, topic }) => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
                    var _b;
                    let consumeEndHooks = [];
                    let consumeErrorHooks = [];
                    let consumeSuccessHooks = [];
                    if (consumeHooks.length) {
                        yield Promise.all(consumeHooks.map((hook) => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
                            const foo = yield hook({
                                topic,
                                metadata: baggage.metadata,
                                fullData: baggage.payload,
                                documentNode: this.consumeValidator.getDocumentForTopic(topic),
                            });
                            if (!foo) {
                                return;
                            }
                            if (foo.consumeEndHook) {
                                consumeEndHooks.push(foo.consumeEndHook);
                            }
                            if (foo.consumeSuccessHook) {
                                consumeSuccessHooks.push(foo.consumeSuccessHook);
                            }
                            if (foo.consumeErrorHook) {
                                consumeErrorHooks.push(foo.consumeErrorHook);
                            }
                        })));
                    }
                    try {
                        const extractedPayload = yield ((_b = this.consumeValidator) === null || _b === void 0 ? void 0 : _b.extractData({
                            topic,
                            data: baggage.payload,
                        }));
                        yield args.cb({
                            topic: topic,
                            payload: extractedPayload,
                            metadata: baggage.metadata,
                            fullData: baggage.payload,
                        });
                        if (consumeSuccessHooks.length) {
                            yield Promise.all(consumeSuccessHooks.map((hook) => {
                                return hook();
                            }));
                        }
                    }
                    catch (e) {
                        if (consumeErrorHooks.length) {
                            yield Promise.all(consumeErrorHooks.map((hook) => {
                                return hook(e);
                            }));
                        }
                        throw e;
                    }
                    finally {
                        if (consumeEndHooks.length) {
                            yield Promise.all(consumeEndHooks.map((hook) => {
                                return hook();
                            }));
                        }
                    }
                }));
            }));
        });
        this.publish = (props) => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            var _c, _d, _e;
            if (!this.publishValidator) {
                throw new Error("Publish config not added!");
            }
            const metadata = Object.assign(Object.assign({ "x-request-id": (0, uuid_1.v4)() }, props.metadata), { messageId: (0, uuid_1.v4)(), publishTime: new Date().toISOString() });
            const publishHooks = ((_c = this.config.plugins) === null || _c === void 0 ? void 0 : _c.map((a) => a.publishStartHook).filter(ts_is_present_1.isPresent)) || [];
            let publishEndHooks = [];
            let publishErrorHooks = [];
            let publishSuccessHooks = [];
            if (publishHooks.length) {
                yield publishHooks.map((hook) => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
                    const r = yield hook({
                        topic: props.topic,
                        payload: props.payload,
                        metadata,
                    });
                    if (!r) {
                        return;
                    }
                    if (r.publishEndHook) {
                        publishEndHooks.push(r.publishEndHook);
                    }
                    if (r.publishSuccessHook) {
                        publishSuccessHooks.push(r.publishSuccessHook);
                    }
                    if (r.publishErrorHook) {
                        publishErrorHooks.push(r.publishErrorHook);
                    }
                }));
            }
            try {
                yield this.publishValidator.publishValidate(props);
                yield ((_d = this.config.publisher) === null || _d === void 0 ? void 0 : _d.publish({
                    topic: props.topic,
                    baggage: {
                        metadata,
                        payload: props.payload,
                    },
                }));
                if (publishSuccessHooks.length) {
                    yield Promise.all(publishSuccessHooks.map((hook) => {
                        return hook();
                    }));
                }
            }
            catch (e) {
                if (publishErrorHooks.length) {
                    yield Promise.all(publishErrorHooks.map((hook) => {
                        return hook(e);
                    }));
                }
                if (e instanceof _1.InvalidPublishTopic &&
                    ((_e = this.config.publisher) === null || _e === void 0 ? void 0 : _e.allowInvalidTopic)) {
                    return;
                }
                throw e;
            }
            finally {
                if (publishEndHooks.length) {
                    yield Promise.all(publishEndHooks.map((hook) => {
                        return hook();
                    }));
                }
            }
            return;
        });
        if (config.publisher) {
            this.publishValidator = new EventBusValidator_1.EventBusValidator({
                publisherSchema: config.publisher.schema,
            });
        }
        if (config.subscriber) {
            this.consumeValidator = new EventBusValidator_1.EventBusValidator({
                publisherSchema: config.subscriber.schema,
            });
        }
    }
}
exports.VanillaEventBus = VanillaEventBus;
